/*  HSYCONET.cpp - HSYCO Library for Arduino
	Copyright (C) 2012-2013 Home Systems Consulting S.p.A. - All rights reserved.

	For information, see the Home Systems Consulting web site:
	http://www.homesystemsconsulting.com/
	
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
    See file LICENSE.txt for further informations on licensing terms.
*/

#include "HSYCONET.h"
#include <Ethernet.h>
#include <SPI.h>


extern "C" {
#include <string.h>
#include <stdlib.h>
}

/* constructor */
HSYCONETClass::HSYCONETClass(EthernetServer &s) : HSYCOServer(s) {
	currentCallback = NULL;
	init();
}

/* public methods */
void HSYCONETClass::begin() {
	HSYCOServer.begin();
}

void HSYCONETClass::debug(void) {
	debugMode = true;
}

void HSYCONETClass::process(void) {
	EthernetClient client = HSYCOServer.available();
	if (client) {
		while (client.available()) {
			unsigned char inChar = (char)client.read();
			if (debugMode || inputStateStart) {
				if (inChar == '\n') {
					inputStateStart = false;
					inputBufferCommit();
					if (inputValueBufferLength == 0 && strcmp(inputNameBuffer, "ping") == 0) {
						if (!debugMode) {
							HSYCOServer.write(255);
						}
						HSYCOServer.write("ping\n");
					} else if (inputValueBufferLength == 0 && strcmp(inputNameBuffer, "version") == 0) {
						if (!debugMode) {
							HSYCOServer.write(255);
						}
						HSYCOServer.write("V.");
						HSYCOServer.write(HSYCO_VERSION);
						HSYCOServer.write('\n');
					} else if (strcmp(inputNameBuffer, "pinmode") == 0) {
						String params = String(inputValueBuffer);
						int sep = params.indexOf(',');
						if (sep > 0) {
							inputValueBuffer[sep] = '\0';
							char *mode = inputValueBuffer + sep + 1;
							char *p;
							int pin = (inputValueBuffer[0] == 'a') ? A0 + (int)strtol(inputValueBuffer + 1, &p, 10) : (int)strtol(inputValueBuffer, &p, 10);
							if (!*p) {
								if (strcmp(mode, "input") == 0) {
									pinMode(pin, INPUT);
								} else if (strcmp(mode, "output") == 0) {
									pinMode(pin, OUTPUT);
								} else if (strcmp(mode, "input_pullup") == 0) {
									pinMode(pin, INPUT_PULLUP);
								}
							}
						}
					} else if (strcmp(inputNameBuffer, "digitalwrite") == 0) {
						String params = String(inputValueBuffer);
						int sep = params.indexOf(',');
						if (sep > 0) {
							inputValueBuffer[sep] = '\0';
							char *mode = inputValueBuffer + sep + 1;
							char *p;
							int pin = (inputValueBuffer[0] == 'a') ? A0 + (int)strtol(inputValueBuffer + 1, &p, 10) : (int)strtol(inputValueBuffer, &p, 10);
							if (!*p) {
								if (strcmp(mode, "1") == 0) {
									digitalWrite(pin, HIGH);
									if (!debugMode) {
										HSYCOServer.write(255);
									}
									HSYCOServer.print("pin.");
									HSYCOServer.print(inputValueBuffer);
									HSYCOServer.print("=1\n");
								} else if (strcmp(mode, "0") == 0) {
									digitalWrite(pin, LOW);
									if (!debugMode) {
										HSYCOServer.write(255);
									}
									HSYCOServer.print("pin.");
									HSYCOServer.print(inputValueBuffer);
									HSYCOServer.print("=0\n");
								}
							}
						}
					} else if (strcmp(inputNameBuffer, "digitalread") == 0) {
						char *p;
						int pin = (inputValueBuffer[0] == 'a') ? A0 + (int)strtol(inputValueBuffer + 1, &p, 10) : (int)strtol(inputValueBuffer, &p, 10);
						if (!*p) {
							if (!debugMode) {
								HSYCOServer.write(255);
							}
							HSYCOServer.print("pin.");
							HSYCOServer.print(inputValueBuffer);
							if (digitalRead(pin) == HIGH) {
								HSYCOServer.print("=1\n");
							} else {
								HSYCOServer.print("=0\n");
							}
						}
					} else if (strcmp(inputNameBuffer, "analogwrite") == 0) {
						String params = String(inputValueBuffer);
						int sep = params.indexOf(',');
						if (sep > 0) {
							int value = atoi(inputValueBuffer + sep + 1);
							inputValueBuffer[sep] = '\0';
							char *p;
							int pin;
							if (strncmp(inputValueBuffer, "dac", 3) == 0) {
								if (HSYC0_DAC0 != -1) {
									pin =  HSYC0_DAC0 + (int)strtol(inputValueBuffer + 3, &p, 10);
								}
							} else if (inputValueBuffer[0] == 'a') {
								pin =  A0 + (int)strtol(inputValueBuffer + 1, &p, 10);
							} else {
								pin =  (int)strtol(inputValueBuffer, &p, 10);
							}
							if (!*p) {
								if (!debugMode) {
									HSYCOServer.write(255);
								}
								analogWrite(pin, value);
								HSYCOServer.print("pin.");
								HSYCOServer.print(inputValueBuffer);
								HSYCOServer.print('=');
								HSYCOServer.print(value);
								HSYCOServer.print('\n');
							}
						}
					} else if (strcmp(inputNameBuffer, "analogread") == 0) {
						char *p;
						int pin = (inputValueBuffer[0] == 'a') ? A0 + (int)strtol(inputValueBuffer + 1, &p, 10) : (int)strtol(inputValueBuffer, &p, 10);
						if (!*p) {
							if (!debugMode) {
								HSYCOServer.write(255);
							}
							HSYCOServer.print("pin.");
							HSYCOServer.print(inputValueBuffer);
							HSYCOServer.print('=');
							HSYCOServer.print(analogRead(pin));
							HSYCOServer.print('\n');
						}
					} else if (inputStateName && currentCallback != NULL && strncmp(inputNameBuffer, "call:", 5) == 0) {
						(*currentCallback)(inputNameBuffer + 5, inputValueBuffer);
					}
					inputBufferReset();
				} else if (inChar == '=') {
					inputBufferCommit();
					inputStateName = true;
				} else if (inChar != '\r') {
					inputBufferAppend(inChar);
				}
			} else if (inChar == 255) {
				inputStateStart = true;
			}
		}
	}
}

void HSYCONETClass::ioSet(char* name, char* value) {
	if (!debugMode) {
		HSYCOServer.write(255);
	}
	HSYCOServer.write(name);
	HSYCOServer.write('=');
	HSYCOServer.write(value);
	HSYCOServer.write('\n');
}

void HSYCONETClass::ioSet(char* name, int value) {
	if (!debugMode) {
		HSYCOServer.write(255);
	}
	HSYCOServer.write(name);
	HSYCOServer.write('=');
	HSYCOServer.print(value);
	HSYCOServer.write('\n');
}

void HSYCONETClass::attach(callbackFunction myFunction) {
	currentCallback = myFunction;
}

void HSYCONETClass::detach(void) {
	currentCallback = NULL;
}

/* private methods */
void HSYCONETClass::init(void) {
	inputBufferInit();
}

void HSYCONETClass::inputBufferInit(void) {
	inputStateName = false;
	inputNameBufferSize = 33;
	inputValueBufferSize = 33;
	inputNameBufferLength = 0;
	inputValueBufferLength = 0;
	inputNameBuffer = (char*)malloc(inputNameBufferSize);
	inputValueBuffer = (char*)malloc(inputValueBufferSize);
}

void HSYCONETClass::inputBufferReset(void) {
	inputStateName = false;
	inputNameBufferLength = 0;
	inputValueBufferLength = 0;
}

void HSYCONETClass::inputBufferAppend(char c) {
	if (!inputStateName) {
		if (inputNameBufferLength + 1 >= inputNameBufferSize) {
			inputNameBufferSize += 32;
			inputNameBuffer = (char*)realloc(inputNameBuffer, inputNameBufferSize);
		}
		inputNameBuffer[inputNameBufferLength] = c;
		inputNameBufferLength++;
	} else {
		if (inputValueBufferLength + 1 >= inputValueBufferSize) {
			inputValueBufferSize += 32;
			inputValueBuffer = (char*)realloc(inputValueBuffer, inputValueBufferSize);
		}
		inputValueBuffer[inputValueBufferLength] = c;
		inputValueBufferLength++;
	}
}

void HSYCONETClass::inputBufferCommit(void) {
	if (!inputStateName) {
		inputNameBuffer[inputNameBufferLength] = '\0';
		inputValueBuffer[0] = '\0';
	} else {
		inputValueBuffer[inputValueBufferLength] = '\0';
	}
}

// create the instance
EthernetServer server(23);
HSYCONETClass HSYCONET(server);
